# -*- coding:utf-8 -*-
from dataclasses import dataclass

import pytest

from ysql.entity import _parse_attr_value, _parse_constraints, Entity, Constraint as cs, _Constraint


@Entity
@dataclass
class Student:
    attr_int: int
    attr_float: float
    attr_bool: bool
    attr_str: str

    attr_tuple: tuple
    attr_list = list
    attr_dict = dict
    attr_set = set

    attr_int_default: int = 1
    attr_float_default: float = 1.0
    attr_bool_default: bool = True
    attr_str_default: str = '1'

    attr_1cs: str = cs.not_null
    attr_2cs: str = cs.not_null, cs.unique
    attr_3cs: str = cs.not_null, cs.unique, cs.primary_key

    attr_str_and_cs: str = '1', cs.not_null
    attr_int_and_cs: int = 1, cs.not_null
    attr_float_and_cs: float = 1.0, cs.not_null
    attr_bool_and_cs: bool = True, cs.not_null

    attr_str_and_2cs: str = '1', cs.not_null, cs.unique
    attr_int_and_2cs: int = 1, cs.not_null, cs.unique
    attr_float_and_2cs: float = 1.0, cs.not_null, cs.unique
    attr_bool_and_2cs: bool = True, cs.not_null, cs.unique

    id: int = cs.auto_primary_key

    attr_tuple_c: tuple = cs.ignore
    attr_list_c: list = cs.ignore
    attr_dict_c: dict = cs.ignore
    attr_set_c: set = cs.ignore

    attr_tuple_c_and_v: tuple = cs.ignore, (1,)
    attr_tuple2_c_and_v: tuple = cs.ignore, 1, 2
    attr_list_c_and_v: list = cs.ignore, [1]
    attr_dict_c_and_v: dict = cs.ignore, {'1': 1}
    attr_set_c_and_v: set = cs.ignore, {1}


def test_parse_1value():
    assert Student.attr_int_default == _parse_attr_value(Student.attr_int_default)
    assert Student.attr_float_default == _parse_attr_value(Student.attr_float_default)
    assert Student.attr_bool_default == _parse_attr_value(Student.attr_bool_default)
    assert Student.attr_str_default == _parse_attr_value(Student.attr_str_default)


def test_parse_1v_from_1c():
    assert Student.attr_int_default == _parse_attr_value(Student.attr_int_and_cs)
    assert Student.attr_float_default == _parse_attr_value(Student.attr_float_and_cs)
    assert Student.attr_bool_default == _parse_attr_value(Student.attr_bool_and_cs)
    assert Student.attr_str_default == _parse_attr_value(Student.attr_str_and_cs)


def test_parse_1v_from_2c():
    assert Student.attr_int_default == _parse_attr_value(Student.attr_int_and_2cs)
    assert Student.attr_float_default == _parse_attr_value(Student.attr_float_and_2cs)
    assert Student.attr_bool_default == _parse_attr_value(Student.attr_bool_and_2cs)
    assert Student.attr_str_default == _parse_attr_value(Student.attr_str_and_2cs)


def test_parse_1constraint():
    assert isinstance(Student.attr_1cs, _Constraint)
    assert _parse_constraints(Student.attr_1cs) == list(cs.not_null.constraint)
    assert _parse_constraints(Student.id) == list(cs.auto_primary_key.constraint)


def test_parse_2c():
    constraints = {cs.not_null.constraint[0], cs.unique.constraint[0]}
    assert set(_parse_constraints(Student.attr_2cs)) == constraints


def test_parse_3c():
    constraints = {cs.not_null.constraint[0], cs.unique.constraint[0], cs.primary_key.constraint[0]}
    assert set(_parse_constraints(Student.attr_3cs)) == constraints


def test_parse_value_from_c_no_default():
    assert _parse_attr_value(Student.attr_1cs) is None
    assert _parse_attr_value(Student.attr_2cs) is None
    assert _parse_attr_value(Student.attr_3cs) is None


def test_parse_1c_from_1v():
    assert _parse_constraints(Student.attr_str_and_cs) == list(cs.not_null.constraint)
    assert _parse_constraints(Student.attr_int_and_cs) == list(cs.not_null.constraint)
    assert _parse_constraints(Student.attr_bool_and_cs) == list(cs.not_null.constraint)
    assert _parse_constraints(Student.attr_float_and_cs) == list(cs.not_null.constraint)


def test_parse_2c_from_1v():
    constraints = {cs.not_null.constraint[0], cs.unique.constraint[0]}
    assert set(_parse_constraints(Student.attr_str_and_2cs)) == constraints
    assert set(_parse_constraints(Student.attr_int_and_2cs)) == constraints
    assert set(_parse_constraints(Student.attr_bool_and_2cs)) == constraints
    assert set(_parse_constraints(Student.attr_float_and_2cs)) == constraints


def test_parse_ignore_constraint():
    assert _parse_constraints(Student.attr_tuple_c) == list(cs.ignore.constraint)
    assert _parse_constraints(Student.attr_list_c) == list(cs.ignore.constraint)
    assert _parse_constraints(Student.attr_dict_c) == list(cs.ignore.constraint)
    assert _parse_constraints(Student.attr_set_c) == list(cs.ignore.constraint)


def test_parse_ignore_value():
    assert _parse_attr_value(Student.attr_tuple_c_and_v) == (1,)
    assert _parse_attr_value(Student.attr_tuple2_c_and_v) == (1, 2)
    assert _parse_attr_value(Student.attr_list_c_and_v) == [1]
    assert _parse_attr_value(Student.attr_dict_c_and_v) == {'1': 1}
    assert _parse_attr_value(Student.attr_set_c_and_v) == {1}


def test_parse_ignore_value_no_default():
    assert _parse_attr_value(Student.attr_tuple_c) is None
    assert _parse_attr_value(Student.attr_list_c) is None
    assert _parse_attr_value(Student.attr_dict_c) is None
    assert _parse_attr_value(Student.attr_set_c) is None


if __name__ == '__main__':
    pytest.main(["-vv",
                 "--capture=no",
                 __file__])



# -*- coding:utf-8 -*-
import pytest


def test_create_tables(init_db):
    """建库"""
    db = init_db
    db.create_tables()


def test_create_with_comment(init_db):
    """注释功能"""
    db = init_db
    db.dao_student_info._create_table()


if __name__ == '__main__':
    pytest.main(["-v", __file__])



# -*- coding:utf-8 -*-
import os
from dataclasses import dataclass

import pytest

from ysql import Dao, MetaDatabase
from ysql.entity import Entity, Constraint as cs


@Entity
@dataclass
class Student:
    name: str
    score: list = cs.ignore


@Dao(Student)
class DaoStudent:
    pass


class Database(MetaDatabase):
    dao = DaoStudent()


def test_entity_use_list():
    db_path = f"db_folder/{os.path.basename(__file__).split('.')[0]}.db"
    if os.path.exists(db_path):
        os.remove(db_path)

    db = Database(db_path)
    db.connect()
    db.create_tables()


if __name__ == '__main__':
    pytest.main(["-vv",
                 "--capture=no",
                 __file__])



# -*- coding:utf-8 -*-
import os
import sqlite3
from dataclasses import dataclass
from typing import Optional

import pytest
from ysql import Entity, Constraint as cs, Dao, DaoFunc, Insert, MetaDatabase
from ysql.dao import Conflict, Sql


@Entity(check_type=True)
@dataclass
class Student:
    name: str = cs.unique
    age: int = None
    id: int = cs.auto_primary_key


@Dao(Student, return_type=Student)
class DaoStudent(DaoFunc):

    @Insert(conflict=Conflict.error)
    def insert_error(self):
        pass

    @Insert(conflict=Conflict.ignore)
    def insert_ignore(self):
        pass

    @Insert(conflict=Conflict.replace)
    def insert_replace(self):
        pass

    @Sql("select * from __ where name=? limit 1;")
    def get_student_by_name(self, name) -> Optional[Student]:
        pass


class Database(MetaDatabase):
    dao_student = DaoStudent()


students = [Student(name='张三', age=10),
            Student(name='张三', age=11),
            Student(name='李四', age=12)]


def init_db() -> Database:
    db_path = "db_folder/test_insert_conflict.db"
    if os.path.exists(db_path):
        os.remove(db_path)

    db = Database(db_path)
    db.connect()
    db.create_tables()
    return db


def test_insert_default():
    db = init_db()
    with pytest.raises(sqlite3.IntegrityError) as exc_info:
        for student in students:
            db.dao_student.insert(student)
    assert "UNIQUE constraint failed" in str(exc_info.value)
    db.rollback()
    db.disconnect()


def test_insert_error():
    db = init_db()
    with pytest.raises(sqlite3.IntegrityError) as exc_info:
        for student in students:
            db.dao_student.insert_error(student)
    assert "UNIQUE constraint failed" in str(exc_info.value)
    db.rollback()
    db.disconnect()


def test_insert_ignore():
    db = init_db()
    for student in students:
        db.dao_student.insert_ignore(student)
    db.commit()

    select_student = students[0]
    student_conflict = db.dao_student.get_student_by_name(name=select_student.name)
    assert student_conflict.age == select_student.age
    db.disconnect()


def test_insert_replace():
    db = init_db()
    for student in students:
        db.dao_student.insert_replace(student)
    db.commit()

    select_student = students[0]
    student_conflict = db.dao_student.get_student_by_name(name=select_student.name)
    assert student_conflict.age == students[1].age
    db.disconnect()


if __name__ == '__main__':
    pytest.main(["-vv", "--capture=no", __file__])



# -*- coding: utf-8 -*-
from dataclasses import dataclass
from typing import Union


# ====================================================================================================================
# 模块常量
@dataclass
class _Constraint:
    """内部维护的约束类，存储实际约束"""
    constraint: tuple


_PRIMARY_KEY = _Constraint(constraint=("PRIMARY KEY",))
_AUTO_PRIMARY_KEY = _Constraint(constraint=("PRIMARY KEY AUTOINCREMENT",))
_NOT_NULL = _Constraint(constraint=("NOT NULL",))
_UNIQUE = _Constraint(constraint=("UNIQUE",))
_IGNORE = _Constraint(constraint=("ignore",))

_NO_ACTION = 'NO ACTION'
_RESTRICT = 'RESTRICT'
_CASCADE = 'CASCADE'
_SET_NULL = 'SET NULL'
_SET_DEFAULT = 'SET DEFAULT'

_ENABLED_TYPES = (int, str, float, bytes, bool, type(None))
_ENABLED_UNION = Union[_ENABLED_TYPES]
_ENTITY_INIT = "__entity_init__"


# ====================================================================================================================
def Entity(check_type: bool = False):
    """数据类dataclass的装饰器。

    在定义dataclass时，同时将约束条件和默认值赋予类属性。
    通过改造原生dataclass的init方法和getattribute方法，实现sql建表时约束条件的解析，以及正常使用被装饰dataclass时属性值的获取。

    Args:
        check_type(bool): 若传入True，实例化数据类时会检查属性值类型和注释的类型是否一致，不一致将触发类型错误。

    Example1:
        @Entity
        @dataclass
        class Student:  # 类名即表名，不区分大小写
            name: str  # 字段名以及字段类型
            score: float = 100

    Example2:
        @Entity(check_type=True)  # 启用类型检查
        @dataclass
        class Student:
            name: str
            score: float = 100.0
            # Example1中的score属性虽然注释为float，实际默认值为int，但仍然可以正常工作（由于python和sqlite都是宽松的类型约束）。

    !Note:
        在实用角度上@dataclass应该合并到@Entity内部，这样定义数据类时只需要使用一个装饰器，并且不需要使用者关心什么是dataclass，
        但经过测试发现，如果不显式的使用@dataclass装饰器，在实例化数据类时Pycharm将无法给出代码提示，这是不可忍受的。
        并且只有Pycharm2020及之前的版本可以正确给出代码提示，高于2020版存在代码提示的bug，详见:
        https://intellij-support.jetbrains.com/hc/en-us/community/posts/4421575751442-Code-Completion-doesn-t-work-for-class-functions-decorated-with-decorators-that-return-inner-functions

    """

    def decorator(cls):
        orig_get_attr = cls.__getattribute__
        orig_init = cls.__init__

        def get_attr(self, name):
            """重写被装饰dataclass的取值方法"""
            # 只有访问定义的属性时才进一步处理
            orig_value = orig_get_attr(self, name)
            if name in set(attr_name for attr_name, attr_type in cls.__annotations__.items()):
                return _parse_attr_value(orig_value)
            # 访问其他属性则返回原始值
            return orig_value

        def init_and_check_type(self, *args, **kwargs):
            """重写被装饰dataclass的初始化方法"""
            orig_init(self, *args, **kwargs)
            for attr_name, attr_type in cls.__annotations__.items():
                # 再检查属性值的类型与类型注解是否一致
                attr_value = get_attr(self, attr_name)
                # 不检查默认值为None的属性
                if attr_value is None:
                    continue
                elif type(attr_value) != attr_type:
                    raise TypeError(
                        f"实例化'{cls.__name__}'类时,"
                        f"'{attr_name}'属性的类型应该是 '{attr_type.__name__}' ,"
                        f"但得到的是 '{type(attr_value).__name__}'")

        # 由于无参装饰器的特性，在无参使用该装饰器时，check_type的值被覆盖为cls，因此必须显式的与True进行判断
        if check_type == True:  # noqa
            cls.__init__ = init_and_check_type
        setattr(cls, _ENTITY_INIT, orig_init)
        cls.__getattribute__ = get_attr
        return cls

    # 无参使用该装饰器时
    if callable(check_type):
        return decorator(cls=check_type)  # noqa

    return decorator


def is_entity(obj) -> bool:
    """判断该对象是否是entity数据类"""
    return hasattr(obj, _ENTITY_INIT)


class Constraint:
    """对外开放的各种字段约束。

    Example:
        @Entity
        @dataclass
        class Student:
            name: str  # 可以不使用约束
            score: float = 100  # 可以只赋予默认值
            address: str = 'HIT', Constraint.not_null  # 同时使用默认值和约束，需要以逗号分隔开，顺序任意
            student_id: int = Constraint.auto_primary_key, Constraint.not_null  # 同时使用多条约束，需要以逗号分隔开

    !Note:
        建议导入Constraint类时使用别名，可以有效简化存在大量约束的使用场景。

        Example:
            from ysql import Constraint as cs

            @Entity
            @dataclass
            class Student:
                name: str
                score: float = 100
                address: str = 'HIT', cs.not_null
                student_id: int = cs.auto_primary_key, cs.not_null

    """

    # =================================================================================================================
    # 1.可直接使用的约束常量
    primary_key = _PRIMARY_KEY  # 主键
    auto_primary_key = _AUTO_PRIMARY_KEY  # 自增主键
    not_null = _NOT_NULL  # 非空
    unique = _UNIQUE  # 唯一

    # 非sql的特殊约束
    ignore = _IGNORE  # 用于建表时忽略某字段/属性

    # 针对外键的约束
    no_action = _NO_ACTION
    cascade = _CASCADE
    set_null = _SET_NULL
    restrict = _RESTRICT
    set_default = _SET_DEFAULT

    # =================================================================================================================
    # 2.需要外部传值的约束
    @staticmethod
    def default(default_value: _ENABLED_UNION):
        """默认值约束

        Args:
            default_value: 该字段在sql中的默认值，与定义数据类时使用默认值作用类似。

        Example:
            @Entity
            @dataclass
            class Student:
                name: str
                score1: float = 100
                score2: float = Constraint.default(100)  # 与score1作用类似
                student_id: int = Constraint.auto_primary_key

        """
        if type(default_value) in _ENABLED_TYPES:
            return _Constraint(constraint=(f'DEFAULT {default_value}',))  # noqa
        raise TypeError(
            f"entity数据类属性默认值允许的数据类型: 'int', 'str', 'float', 'bytes',"
            f"但得到的是 '{type(default_value).__name__}'"
        )

    @staticmethod
    def check(check_condition: str):
        """条件约束

        Args:
            check_condition: 具体条件

        Example:
            @Entity
            @dataclass
            class Student:
                name: str
                score: float = Constraint.check('score > 60')  # 需要填写该字段的字符形式名称
                student_id: int = Constraint.auto_primary_key

        """
        if type(check_condition) == str:
            return _Constraint(constraint=(f'CHECK({check_condition})',))  # noqa
        raise TypeError(
            f"对entity数据类属性使用条件约束时，允许的数据类型: 'str',"
            f"但得到的是 '{type(check_condition).__name__}'")

    @staticmethod
    def foreign_key(entity, field, delete_link=None, update_link=None):
        """外键约束

        Args:
            entity: 外键所在的数据类（父表）
            field: 外键对应的数据类属性
            delete_link: 级联删除方式
            update_link: 级联更新方式

        Example:
            @Entity
            @dataclass
            class Student:  # 父表
                name: str = Constraint.not_null
                student_id: int = Constraint.auto_primary_key

            @Entity
            @dataclass
            class Score:  # 子表
                score: float
                score_id: int = Constraint.auto_primary_key
                # 对student_id字段设置外键关联
                student_id: int = Constraint.foreign_key(entity=Student,
                                                         field='student_id',
                                                         delete_link=Constraint.cascade,
                                                         update_link=Constraint.cascade)

        """
        return _Constraint(constraint=((entity.__name__.lower(), field, delete_link, update_link),))  # noqa

    @staticmethod
    def comment(comment: str):
        """字段注释

        Args:
            comment: 具体注释。注意，在sqlite中只能通过DDL(Data Definition Language)查看。

        Example:
            @Entity
            @dataclass
            class Student:
                name: str = Constraint.comment('学生姓名')
                student_id: int = Constraint.auto_primary_key, Constraint.comment('学生id')

        """
        if type(comment) == str:
            # 目前仅支持sqlite的注释格式
            return _Constraint(constraint=(f'-- {comment}\n',))  # noqa
        raise TypeError(
            f"对entity数据类属性使用sql注释时，允许的数据类型: 'str',"
            f"但得到的是 '{type(comment).__name__}'")


# ====================================================================================================================
# 模块方法
def _parse_constraints(attr_value) -> list[str, tuple]:
    """从属性原始值中解析出约束条件。"""
    # 无约束
    constraints = []
    # 是单约束
    if isinstance(attr_value, _Constraint):
        constraints = list(attr_value.constraint)
    # 包含多约束
    elif isinstance(attr_value, tuple):
        for item in attr_value:
            if not isinstance(item, _Constraint):
                continue

            if item in constraints:
                raise TypeError(
                    f"重复使用了约束条件: {item.constraint}")

            constraints.append(item.constraint[0])

    return constraints


def _parse_attr_value(attr_value) -> _ENABLED_UNION:
    """从属性原始值中过滤约束条件，解析出真正的属性值。并不负责检查值是否匹配类型。"""
    # 单个约束
    if isinstance(attr_value, _Constraint):
        return None
    # 多约束
    elif isinstance(attr_value, tuple):
        parsed_value = []

        for item in attr_value:
            if isinstance(item, _Constraint):
                continue
            parsed_value.append(item)

        # 仅多个约束
        if len(parsed_value) == 0:
            return None
        # 包含单个值
        elif len(parsed_value) == 1:
            return parsed_value[0]
        # tuple类型的值
        return tuple(parsed_value)

    return attr_value



import setuptools

with open("README_pypi.md", "r", encoding='utf-8') as fh:
    long_description = fh.read()

setuptools.setup(
    name="ysql",
    version="1.4.12",
    author="dfqxhhh",
    author_email="dfqxhhh@163.com",
    description="a more efficient and concise SQLite framework.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://gitee.com/darlingxyz/ysql",
    packages=['ysql'],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
)






# -*- coding:utf-8 -*-
import time
from dataclasses import asdict

import pytest

from test.model import Student, Score, Database, Student2

# 测试用例
test_cases = [
    ("SELECT column1 FROM table1", True),  # 仅选择一个字段
    ("SELECT column1, column2 FROM table1", False),  # 选择多个字段
    ("SELECT column1, column2, column3 FROM table1", False),  # 选择多个字段
    ("SELECT DISTINCT column1 FROM table1", True),  # 使用 DISTINCT
    ("SELECT column1 FROM table1 WHERE column2 = 42", True),  # 包含 WHERE 子句
    ("SELECT column1 FROM table1 JOIN table2 ON table1.id = table2.id", True),  # 包含 JOIN
    ("SELECT column1 FROM (SELECT column2 FROM table1) AS subquery", True),  # 包含子查询
    ("SELECT * FROM table1", False),  # 选择所有字段
    ("SELECT COUNT(*) FROM table1", False),  # 使用聚合函数
    ("SELECT column1, column2 FROM table1 WHERE column3 = (SELECT MAX(column4) FROM table2)", False),  # 复杂查询
    ("INSERT INTO table1 (column1, column2) VALUES (value1, value2)", False),  # 插入语句
    ("UPDATE table1 SET column1 = value1 WHERE column2 = value2", False),  # 更新语句
    ("DELETE FROM table1 WHERE column1 = value1", False),  # 删除语句
    ("SELECT MIN(column1) FROM table1", False),  # 使用 MIN() 聚合函数
    ("SELECT MAX(column1) FROM table1", False),  # 使用 MAX() 聚合函数
    ("SELECT AVG(column1) FROM table1", False),  # 使用 AVG() 聚合函数
    ("SELECT SUM(column1) FROM table1", False),  # 使用 SUM() 聚合函数
    ("SELECT column1, column2 FROM table1 GROUP BY column1", False),  # 使用 GROUP BY 子句
    ("SELECT column1 FROM table1 HAVING COUNT(*) > 10", False),  # 使用 HAVING 子句
]


def generate_students(start, end):
    return [
        Student(name=f'李华{i}', age=i, phone=123456789,
                weight=50.0, height=100.0 + i, address=f'hit{i}',
                student_id=i)
        for i in range(start, end)
    ]


student_num = 100
students = [
    Student(name=f'李华{i + 1}', age=i + 1, phone=123456789,
            weight=50.0, height=100.0 + i + 1, address=f'hit{i + 1}',
            student_id=i + 1)
    for i in range(0, student_num)
]


@pytest.mark.order(1)
def test_dao_create_table(init_db: Database):
    """创建表"""
    db = init_db
    db.dao_student._create_table()
    db.dao_score._create_table()
    db.dao_student_info._create_table()
    db.commit()


@pytest.mark.order(2)
def test_insert(init_db):
    """插入数据"""
    db = init_db
    for student in students:
        record_id = db.dao_student.insert(entity=student)
        db.commit()

        result = db.dao_student.select_student_by_id(record_id)
        result = result[0]
        assert asdict(result) == asdict(students[result.student_id - 1])


def test_select_all(init_db):
    """查询全部"""
    db = init_db
    results = db.dao_student.select_all()

    for result in results:
        assert asdict(result) == asdict(students[result.student_id - 1])


def test_select_all_with_substitute(init_db):
    """使用表名替代符"""
    db = init_db
    results = db.dao_student.select_all_with_substitute()

    for result in results:
        assert asdict(result) == asdict(students[result.student_id - 1])


def test_select_student_by_id(init_db):
    """具体查询"""
    db = init_db
    select_id = 50
    result = db.dao_student.select_student_by_id(student_id=select_id)
    assert asdict(result[0]) == asdict(students[select_id - 1])


def test_update_name(init_db):
    """更新"""
    db = init_db
    select_id = 50
    new_name = '好家伙'
    db.dao_student.update_name_by_id(name=new_name, student_id=select_id)
    db.commit()
    result = db.dao_student.select_student_by_id(select_id)
    assert result[0].name == new_name


def test_update_name_with_substitute(init_db):
    """使用表名替代符更新"""
    db = init_db
    select_id = 50
    new_name = '好家伙'
    db.dao_student.update_name_by_id_with_substitute(name=new_name, student_id=select_id)
    db.commit()
    result = db.dao_student.select_student_by_id(select_id)
    assert result[0].name == new_name


def test_multiple_table_substitute(init_db: Database):
    """复杂sql中同时使用多个表名替代符"""
    db = init_db
    result = db.dao_student.select_last_student_with_substitute()
    assert result
    assert result[0].student_id == student_num


def test_foreign_key_cascade(init_db):
    """外键"""
    db = init_db
    db.execute("PRAGMA foreign_keys = ON;")
    select_id = 50
    score = Score(150.0, student_id=select_id)
    db.dao_score.insert(score)
    db.dao_student.delete_student_by_id(student_id=select_id)
    db.commit()

    result = db.dao_score.get_score(select_id)
    assert len(result) == 0


def test_ignore(init_db: Database):
    """数据类的忽略属性在建表时"""
    db = init_db
    db.dao_student2._create_table()
    db.commit()
    db.dao_student2.cursor.execute("PRAGMA table_info(student2);")
    # 获取所有列信息
    columns = db.dao_student2.cursor.fetchall()

    # 提取并打印所有字段名
    columns = set(column[1] for column in columns)
    entity_fields = set(attr_name for attr_name, attr_type in Student2.__annotations__.items())
    ignore_fields = {'score', 'address'}
    entity_fields_without_ignore = set(filter(lambda item: item not in ignore_fields, entity_fields))

    assert entity_fields != columns
    assert entity_fields_without_ignore == columns


def test_insert_ignore(init_db: Database):
    """数据类的忽略属性在插入记录时"""
    db = init_db
    student2 = Student2(name='张三', score=100, address='hit')
    db.dao_student2.insert(student2)
    db.commit()


def test_insert_many(init_db: Database):
    """批量插入"""
    db = init_db
    start = 101
    end = 200
    students2 = generate_students(start=start, end=end)
    db.dao_student.insert(entity=students2)
    db.commit()

    for i in range(start, end):
        result = db.dao_student.select_student_by_id(i)
        result = result[0]
        assert asdict(result) == asdict(students2[i - start])


# ====================================================================================================================

def test_fetchone(init_db: Database):
    """查询单记录"""
    db = init_db
    select_id = 90
    result1 = db.dao_student.select_student_by_id(select_id)
    result2 = db.dao_student.select_student_by_id_limit1(select_id)
    assert isinstance(result1, list)
    assert asdict(result1[0]) == asdict(result2)


def test_fetchone_no_record(init_db: Database):
    """查询不存在的单记录"""
    db = init_db
    select_id = 90
    db.dao_student.delete_student_by_id(student_id=select_id)
    result1 = db.dao_student.select_student_by_id(select_id)
    result2 = db.dao_student.select_student_by_id_limit1(select_id)
    assert not result1
    assert result2 is None


# ====================================================================================================================

def test_execute_sql(init_db: Database):
    """执行sql方法"""
    db = init_db
    select_id = 10
    result = db.dao_student.select_one_student(student_id=select_id)
    assert isinstance(result, list)
    assert list
    assert asdict(result[0]) == asdict(students[select_id - 1])


def test_execute_sql_with_substitute(init_db: Database):
    """执行sql方法，使用替代符"""
    db = init_db
    select_id = 10
    result = db.dao_student.select_one_student_with_substitute(student_id=select_id)
    assert asdict(result) == asdict(students[select_id - 1])


def test_execute_sql_with_limit(init_db: Database):
    """执行sql方法，使用limit"""
    db = init_db
    select_id = 10
    result = db.dao_student.select_one_student_with_limit(student_id=select_id)
    assert asdict(result) == asdict(students[select_id - 1])


def test_execute_sql_with_substitute_and_limit(init_db: Database):
    """执行sql方法，使用替代符和limit"""
    db = init_db
    select_id = 10
    result = db.dao_student.select_one_student_with_substitute_and_limit(student_id=select_id)
    assert asdict(result) == asdict(students[select_id - 1])


# ====================================================================================================================

def test_execute_sql_without_dao_func(init_db: Database):
    """执行sql方法"""
    db = init_db
    select_id = 10
    result = db.dao_student3.select_one_student(student_id=select_id)
    assert isinstance(result, list)
    assert list
    assert asdict(result[0]) == asdict(students[select_id - 1])


def test_execute_sql_with_substitute_without_dao_func(init_db: Database):
    """执行sql方法，使用替代符"""
    db = init_db
    select_id = 10
    result = db.dao_student3.select_one_student_with_substitute(student_id=select_id)
    assert asdict(result) == asdict(students[select_id - 1])


def test_execute_sql_with_limit_without_dao_func(init_db: Database):
    """执行sql方法，使用limit"""
    db = init_db
    select_id = 10
    result = db.dao_student3.select_one_student_with_limit(student_id=select_id)
    assert asdict(result) == asdict(students[select_id - 1])


def test_execute_sql_with_substitute_and_limit_without_dao_func(init_db: Database):
    """执行sql方法，使用替代符和limit"""
    db = init_db
    select_id = 10
    result = db.dao_student3.select_one_student_with_substitute_and_limit(student_id=select_id)
    assert asdict(result) == asdict(students[select_id - 1])


def test_execute_sql_with_conditions(init_db: Database):
    """执行sql方法，使用替代符和limit"""
    db = init_db
    select_id = 10
    result = db.dao_student.select_student_with_conditions(student_id=select_id, age=select_id)
    assert asdict(result) == asdict(students[select_id - 1])


# ====================================================================================================================
def test_row_factory(init_db: Database):
    db = init_db
    start = time.time()
    students = generate_students(1000, 100000)
    print(f'\n生成耗时：{time.time() - start}')

    start = time.time()
    db.dao_student.insert(students)
    db.commit()
    end = time.time() - start
    print(f'插入耗时：{end}')

    # 默认数据类
    start = time.time()
    results2 = db.dao_student.select_all2()
    end = time.time() - start
    print(f'entity_auto耗费时间：{end}, 查询结果数量；{len(results2)}')

    start = time.time()
    results3 = db.dao_student.select_all_full()
    end = time.time() - start
    print(f'entity_full_fast耗费时间：{end}, 查询结果数量；{len(results3)}')

    start = time.time()
    results4 = db.dao_student.select_all_part()
    end = time.time() - start
    print(f'entity_part耗费时间：{end}, 查询结果数量；{len(results4)}')

    start = time.time()
    results5 = db.dao_student.select_all_full_no_order()
    end = time.time() - start
    print(f'entity_full_not_order耗费时间：{end}, 查询结果数量；{len(results5)}')

    start = time.time()
    result6 = db.dao_student.select_all_part_and_no_order()
    end = time.time() - start
    print(f'entity_part_and_no_order耗费时间：{end}, 查询结果数量；{len(results4)}')
    assert results4 == result6
    assert results2 == results3 == results5

    start = time.time()
    results_dataclass = db.dao_student.select_all()
    end = time.time() - start
    print(f'dataclass耗费时间：{end}, 查询结果数量；{len(results_dataclass)}')

    start = time.time()
    results_dict = db.dao_student_dict.select_all()
    end = time.time() - start
    print(f'dict耗费时间：{end}, 查询结果数量；{len(results_dict)}')

    start = time.time()
    results_tuple = db.dao_student_tuple.select_all()
    end = time.time() - start
    print(f'tuple耗费时间：{end}, 查询结果数量；{len(results_tuple)}')

    start = time.time()
    results_namedtuple = db.dao_student.select_all_namedtuple()
    end = time.time() - start
    print(f'namedtuple耗费时间：{end}, 查询结果数量；{len(results_namedtuple)}')

    for item_dataclass, item_dict, item_tuple, item_namedtuple in zip(results_dataclass,
                                                                      results_dict,
                                                                      results_tuple,
                                                                      results_namedtuple):
        assert item_dict == item_namedtuple._asdict()
        assert item_tuple == item_namedtuple
        assert item_namedtuple._asdict() == asdict(item_dataclass)


# def test_row_factory_get_memory(init_db: Database):
#     db = init_db
#     start = time.time()
#     students = generate_students(1000, 1000000)
#     print(f'\n生成耗时：{time.time() - start}')
#
#     tracemalloc.start()
#     start = time.time()
#     db.dao_student.insert(students)
#     db.commit()
#     end = time.time() - start
#     current_memory, peak_memory = tracemalloc.get_traced_memory()
#     tracemalloc.stop()
#     print(f'插入耗时：{end}，内存占用：{peak_memory}')
#
#
#     # 默认数据类
#     tracemalloc.start()
#     start = time.time()
#     results = db.dao_student.select_all()
#     end = time.time() - start
#     current_memory, peak_memory = tracemalloc.get_traced_memory()
#     tracemalloc.stop()
#     print(f'dataclass耗费时间：{end}, 查询结果数量；{len(results)}，内存占用：{peak_memory}')
#
#     # 字典
#     tracemalloc.start()
#     start = time.time()
#     results = db.dao_student_dict.select_all()
#     end = time.time() - start
#     current_memory, peak_memory = tracemalloc.get_traced_memory()
#     tracemalloc.stop()
#     print(f'dict耗费时间：{end}, 查询结果数量；{len(results)}，内存占用：{peak_memory}')
#
#     # 元组
#     tracemalloc.start()
#     start = time.time()
#     results = db.dao_student_tuple.select_all()
#     end = time.time() - start
#     current_memory, peak_memory = tracemalloc.get_traced_memory()
#     tracemalloc.stop()
#     print(f'tuple耗费时间：{end}, 查询结果数量；{len(results)}，内存占用：{peak_memory}')
def test_dao_return_type_default_list(init_db: Database):
    db = init_db
    student = students[2]
    result = db.dao_student.select_student_by_id(student.student_id)
    assert result
    assert isinstance(result[0], Student)
    assert result[0] == student


def test_dao_return_type_default(init_db: Database):
    db = init_db
    student = students[2]
    result = db.dao_student.select_student_by_id_limit1(student.student_id)
    assert isinstance(result, Student)
    assert result == student


def test_dao_return_type_dict_list(init_db: Database):
    db = init_db
    student = students[2]
    result = db.dao_student_dict.select_student_by_id2(student.student_id)
    assert result
    assert isinstance(result[0], dict)
    assert result[0] == asdict(student)


def test_dao_return_type_dict(init_db: Database):
    db = init_db
    student = students[2]
    result = db.dao_student_dict.select_student_by_id(student.student_id)
    assert isinstance(result, dict)
    assert result == asdict(student)


def test_dao_return_type_tuple_list(init_db: Database):
    db = init_db
    student = students[2]
    result = db.dao_student_tuple.select_student_by_id2(student.student_id)
    assert result
    assert isinstance(result[0], tuple)
    assert result[0] == tuple(asdict(student).values())


def test_dao_return_type_tuple(init_db: Database):
    db = init_db
    student = students[2]
    result = db.dao_student_tuple.select_student_by_id(student.student_id)
    assert isinstance(result, tuple)
    assert result == tuple(asdict(student).values())


def test_select_single_field(init_db: Database):
    db = init_db
    student = students[3]

    s1 = db.dao_student.select_name_by_id_without_single_field(student.student_id)
    s2 = db.dao_student.select_name_by_id_with_single_field(student.student_id)
    assert s1.name == student.name
    assert s2 == student.name

    result1 = db.dao_student.select_name_by_id_without_single_field_list()
    result2 = db.dao_student.select_name_by_id_with_single_field_list()
    for item1, item2 in zip(result1, result2):
        assert type(item1.name) == str
        assert type(item2) == str


def test_bool(init_db: Database):
    db = init_db
    result1 = db.dao_student.select_bool()
    result2 = db.dao_student.select_bool_no_return_bool()
    assert type(result1) == bool
    assert type(result2.if_aged) == int


def test_sql_fetch_one(init_db: Database):
    db = init_db
    select_id = 3
    result1 = db.dao_student.select_student_by_id(select_id)
    result2 = db.dao_student.select_student_by_id_use_fetch_one(select_id)
    assert type(result1) == list
    assert type(result2) == Student
    assert result1[0] == result2


if __name__ == '__main__':
    pytest.main(["-vv",
                 "--capture=no",
                 __file__])
